#!/usr/bin/wish
# Author: Travis Veldkamp
# Date: December 2002

# These determine the width and height of the cells in the GUI grid
set CELL_SIZEX 40
set CELL_SIZEY 40

# This setting determines the width of the line between cells in the GUI grid
set GRID_WIDTH 3

# Used for GUI spacing
set pad_x 10
set pad_y 6

# These are the GUI colors for a cell that is blank (greenish) or set (black)
set BLANK_COLOR #009966
set BLACK_COLOR #555555

source "LCDgrid.tcl"
source "lcdlib.tcl"

namespace eval ::LCDDraw {
   global CELL_SIZEX CELL_SIZEY GRID_WIDTH

   variable StatusText
   variable can


   ###############
   # BEGIN PROCS #
   ###############


   ################################################################################
   # Send a character map to the LCD                                              #
   ################################################################################
   proc ::LCDDraw::SendMap {CharNum LCDDevice} {
      variable StatusText

      if [LCD::Open $LCDDevice] {
         for {set idx 1} {$idx <= [LCDGrid::GetHeight]} {incr idx} {
            set Byte($idx) [LCDGrid::GetByte $idx]
         }
         LCD::PgmCustom $CharNum $Byte(1) $Byte(2) $Byte(3) $Byte(4) $Byte(5) $Byte(6) $Byte(7) $Byte(8)
         set StatusText "Sent new character to $LCDDevice"
         LCD::Close
      } else {
         set StatusText "Send failed - unable to connect to LCD"
      }

      
   }

   ################################################################################
   # Request that the LCD draw the specified character number                     #
   ################################################################################
   proc ::LCDDraw::DrawChar {CharNum LCDDevice} {

      variable StatusText

      if [LCD::Open $LCDDevice] {
         LCD::SendCustom $CharNum
         LCD::Close
         set StatusText "Drew character on $LCDDevice"
      } else {
         set StatusText "Send failed - unable to connect to LCD"
      }
   }

   ################################################################################
   #                          lcddraw  Initialize                                #
   ################################################################################
   proc ::LCDDraw::Initialize {} {

      LCDGrid::Initialize
   }


   ################################################################################
   #                          lcddraw  Exit                                      #
   ################################################################################
   proc ::LCDDraw::Exit { } {

      destroy .
   }


   ################################################################################
   #                          lcddraw  Save                                      #
   ################################################################################
   proc ::LCDDraw::Save { FileName } {
      variable StatusText

      if [string length $FileName] {
         if [LCDGrid::Save $FileName] {
            set StatusText "Saved to $FileName"
         } else {
            set StatusText "ERROR: failed to save $FileName"
         }
      } else {
         set StatusText "No file chosen - not saved"
      }
   }

   ################################################################################
   # Clear - clear the grid                                                       #
   ################################################################################
   proc ::LCDDraw::Clear {{Fill 0}} {
      variable StatusText

      LCDGrid::Clear $Fill

      ReDraw

      if {$Fill} {
         set StatusText "Filled"
      } else {
         set StatusText "Cleared"
      }

   }

   ################################################################################
   #                          lcddraw  Load                                       #
   ################################################################################
   proc ::LCDDraw::Load { FileName } {
      variable StatusText

      if [LCDGrid::Load $FileName] {
         set StatusText "Loaded from file: $FileName"
         ReDraw
         set StatusText "Done"
      } else {
         set StatusText "ERROR - failed to load file"
      }

   }

   ################################################################################
   # Redraw the screen, based on CellState                                        #
   ################################################################################
   proc ::LCDDraw::ReDraw {} {
      global CELL_SIZEX CELL_SIZEY GRID_WIDTH
      global BLANK_COLOR BLACK_COLOR
      variable can

      # Create the grid of 'pixels'
      for {set yidx 0} {$yidx < [LCDGrid::GetHeight]} {incr yidx} {
         for {set xidx 0} {$xidx < [LCDGrid::GetWidth]} {incr xidx} {
            set x1  [expr (($xidx * $CELL_SIZEX) + (($xidx + 1) * $GRID_WIDTH))]
            set y1  [expr (($yidx * $CELL_SIZEY) + (($yidx + 1) * $GRID_WIDTH))]
            set x2  [expr (($xidx * $CELL_SIZEX) + ($CELL_SIZEX + (($xidx + 1) * $GRID_WIDTH)))]
            set y2  [expr (($yidx * $CELL_SIZEY) + ($CELL_SIZEY + (($yidx + 1) * $GRID_WIDTH)))]
            #puts "$x1,$y1 -> $x2,$y2"
            if {[LCDGrid::GetCellState $xidx $yidx] == 1} {
               set cellcolor $BLACK_COLOR
            } else {
               set cellcolor $BLANK_COLOR
            }
            $can create rect $x1 $y1 $x2 $y2 -width 0 -fill $cellcolor -tag clickable
         }
      }
   }

   ################################################################################
   #                          Click in the canvas                                 #
   ################################################################################
   proc ::LCDDraw::CanvasClick {x y can} {
      global CELL_SIZEX CELL_SIZEY GRID_WIDTH
      global BLACK_COLOR BLANK_COLOR

      #puts "Click: $x, $y"
      set xcell 0
      set x [expr $x - $GRID_WIDTH]
      while {$x > $CELL_SIZEX} {
         incr x [expr ($CELL_SIZEX + $GRID_WIDTH) * -1]
         incr xcell
      }
      if {$xcell >= [LCDGrid::GetWidth]} {
         set xcell [expr [LCDGrid::GetWidth] - 1]
      }
      #puts "X square: $xcell"

      set ycell 0
      set y [expr $y - $GRID_WIDTH]
      while {$y > $CELL_SIZEY} {
         incr y [expr ($CELL_SIZEY + $GRID_WIDTH) * -1]
         incr ycell
      }
      if {$ycell >= [LCDGrid::GetHeight]} {
         set ycell [expr [LCDGrid::GetHeight] - 1]
      }
      #puts "y square: $ycell"

      #puts "CellState Before: [LCDGrid::GetCellState $xcell $ycell]"
      LCDGrid::ToggleCell $xcell $ycell
      #puts "CellState After: [LCDGrid::GetCellState $xcell $ycell]"

      # Now, redraw this cell
      if {[LCDGrid::GetCellState $xcell $ycell] == 1} {
         set cellcolor $BLACK_COLOR
      } else {
         set cellcolor $BLANK_COLOR
      }
      set x1  [expr (($xcell * $CELL_SIZEX) + (($xcell + 1) * $GRID_WIDTH))]
      set y1  [expr (($ycell * $CELL_SIZEY) + (($ycell + 1) * $GRID_WIDTH))]
      set x2  [expr (($xcell * $CELL_SIZEX) + ($CELL_SIZEX + (($xcell + 1) * $GRID_WIDTH)))]
      set y2  [expr (($ycell * $CELL_SIZEY) + ($CELL_SIZEY + (($ycell + 1) * $GRID_WIDTH)))]
      $can create rect $x1 $y1 $x2 $y2 -width 0 -fill $cellcolor -tag clickable

   }

   ####################
   # E N D  P R O C S #
   ####################
   

   ###########################
   # BEGIN GUI Creation Code #
   ###########################

   Initialize

   #if {[winfo exists .]} { destroy .}

   #toplevel .
   #focus .

   set title [label .title -text "LCD Draw" -relief solid -bg #CCCCDD]
   pack $title -ipadx $pad_x -ipady $pad_y -padx $pad_x -pady $pad_y

   # Parent frame for the GUI elements
   set   outlineframe [frame .frame2 -relief groove -bd 2]
   pack $outlineframe -side top

   # Frames for the decode list
   set   Parent     [frame $outlineframe.framedcd -relief groove -bd 2]
   set   BtnFrm     [frame $outlineframe.buttons]

   # Canvas
   set can_width  [expr (([LCDGrid::GetWidth] * $CELL_SIZEX) + (([LCDGrid::GetWidth] + 1) * $GRID_WIDTH))]
   set can_height [expr (([LCDGrid::GetHeight] * $CELL_SIZEY) + (([LCDGrid::GetHeight] + 1) * $GRID_WIDTH))]
   set can [canvas $Parent.canvas -highlightthickness 0 -borderwidth 0 -width $can_width -height $can_height -bg #00FF33]

   # Draw on the cells for the 5x8 matrix...
   ReDraw

   $can bind clickable <Button-1> {::LCDDraw::CanvasClick %x %y %W}
   pack $can 

   set ClearBtn  [button $BtnFrm.clear -text "Clear" -command {LCDDraw::Clear}   -width 10]
   set FillBtn   [button $BtnFrm.fill  -text "Fill"  -command {LCDDraw::Clear 1} -width 10]
   set ExitBtn   [button $BtnFrm.exit  -text "Exit"  -command {LCDDraw::Exit}    -width 10]
   set SendBtn   [button $BtnFrm.send  -text "Send"  -command {LCDDraw::SendMap  $LCDDraw::CharNum $LCDDraw::LCDDev}    -width 10]
   set DrawBtn   [button $BtnFrm.draw  -text "Test"  -command {LCDDraw::DrawChar $LCDDraw::CharNum $LCDDraw::LCDDev}    -width 10]
   pack $Parent $BtnFrm -side left -padx 8
   pack $ClearBtn $FillBtn $SendBtn $DrawBtn $ExitBtn -side top -pady 10

   # Now some entries for the open/save stuff..
   set charframe  [frame .frame25]
   set openframe  [frame .frame3]
   set saveframe  [frame .frame4]

   set CharLabl   [label  $charframe.charlabl -text "Character Number"]
   set DevLabl    [label  $charframe.devlabl  -text "LCD Device"]
   set OpenLabl   [label  $openframe.openlabl -text "Source File"]
   set SaveLabl   [label  $saveframe.savelabl -text "Save To"]
   set CharEnt    [entry  $charframe.charent -textvariable LCDDraw::CharNum   -width 5]
   set DevEnt     [entry  $charframe.devent  -textvariable LCDDraw::LCDDev    -width 15]
   #set OpenEnt    [entry  $openframe.openent -textvariable LCDDraw::InputFile -width 30]
   #set SaveEnt    [entry  $saveframe.saveent -textvariable LCDDraw::SaveFile -width 30]
   #set OpenBrowse [button $openframe.openbrs -text "Browse" -command {set LCDDraw::InputFile [tk_getOpenFile -parent . -defaultextension *.lcd]}]
   #set SaveBrowse [button $saveframe.savebrs -text "Browse" -command {set LCDDraw::SaveFile  [tk_getSaveFile -parent . -defaultextension *.lcd]}]
   set OpenBut    [button $openframe.openbrs -text "Load" -command {LCDDraw::Load [tk_getOpenFile -parent . -defaultextension *.lcd]}]
   set SaveBut    [button $openframe.savebrs -text "Save" -command {LCDDraw::Save [tk_getSaveFile -parent . -defaultextension *.lcd]}]
   #set OpenBut    [button $openframe.openbut -text "Load"   -command {LCDDraw::Load $LCDDraw::InputFile}]
   #set SaveBut    [button $saveframe.savebut -text "Save"   -command {LCDDraw::Save $LCDDraw::SaveFile} ]

   pack $CharLabl $CharEnt $DevLabl $DevEnt -side left -padx 5 -anchor w
   #pack $OpenLabl $OpenEnt $OpenBut $OpenBrowse -side left -padx 5 -anchor w
   #pack $SaveLabl $SaveEnt $SaveBut $SaveBrowse -side left -padx 5 -anchor w
   pack $SaveBut $OpenBut -side left -padx 5 -anchor w
   pack $charframe $openframe $saveframe -side top -pady 5

   set LCDDev "/dev/lcd"
   set CharNum 0


   #Status bar
   set frame4 [frame .frame5]
   pack $frame4 -side top

   label $frame4.status -textvariable LCDDraw::StatusText -justify right
   pack  $frame4.status -side top -padx $pad_x -pady $pad_y -anchor w 

   set StatusText "Welcome User!"
   update

   #try to catch user closing window rather than pressing exit
   wm protocol . WM_DELETE_WINDOW LCDDraw::Exit
   wm title    . "LCD Draw"


} ;#end of namespace


# TLV 11/22/2002
# Profile handler routines
# This works similar to parsing an .ini file
# This file is made of sections.  Each line in a section
# contains a Variable and a Value
#
# Example:
# [FirstSection]
# Temp=c:\work.exe
# .
#
# Comments are any line starting with #
# . denotes the end of a section

package provide Config 1.0

namespace eval ::Config {

   namespace export Destroy IsLoaded GetSectionIndex GetVarCount GetVarName GetValueByIndex \
	                 GetValue GetSectionCount GetSectionName AddSection SectionExists SetVariable \
	                 AddVariable Read Write


   variable VarList
   variable SectionCount
   variable SectionList
   variable IsLoaded
   variable Separator "="
	variable CommentChar "#"

   proc ::Config::Init {} {
      Destroy
   }

   proc ::Config::Destroy {} {
      variable VarList
      variable SectionCount
      variable SectionList
      variable IsLoaded 
      variable Separator 

      set Separator "="

      if [info exists VarList] {
         unset VarList
      }
      if [info exists SectionList] {
         unset SectionList
      }

      set SectionCount 0
      set IsLoaded 0
      #puts "Config destroyed"
   }

   ###########################################################
   # Set the character used to specify comments              #
   ###########################################################
	proc ::Config::SetCommentChar {NewChar} {
		variable CommentChar

		set CommentChar $NewChar
   }

   ###########################################################
   # Get the character used to specify comments              #
   ###########################################################
	proc ::Config::GetCommentChar {} {
		variable CommentChar

		return $CommentChar
   }

   ###########################################################
   # Set the current separator for the config file           #
   # for example, in this example, = is the separator        #
   # [Test Section]                                          #
   # SomeVar=4                                               #
   ###########################################################
   proc ::Config::SetSeparator {NewSep} {
      variable Separator

      set Separator $NewSep
   }

   ###########################################################
   # Return the current separator for the config file        #
   # for example, in this example, = is the separator        #
   # [Test Section]                                          #
   # SomeVar=4                                               #
   ###########################################################
   proc ::Config::GetSeparator {} {
      variable Separator

      return $Separator
   }

   ###########################################################
   # Check if we have read the config from a file yet (Read) #
   ###########################################################
   proc ::Config::IsLoaded {} {
      variable IsLoaded 

      return $IsLoaded
   }

   ###########################################################
   # Given a section name, returns it's index                #
   ###########################################################
   proc ::Config::GetSectionIndex {SectionName} {
      variable SectionList
      variable SectionCount

      set retval -1

      for {set idx 0} {$idx < $SectionCount} {incr idx} {
         if {$SectionList($idx,Name) == $SectionName} {
            set retval $idx
            break
         }
      }

      return $retval

   }

   ###########################################################
   # Returns the count of variables currently loaded         #
   ###########################################################
   proc ::Config::GetVarCount {Section} {
      variable SectionList

      set retval 0
      if [SectionExists $Section] {
         set index [GetSectionIndex $Section]
         set retval $SectionList($index,Count)
      }
      return $retval
   }

   ###########################################################
   # Gets the variable name for the specified section/index  #
   ###########################################################
   proc ::Config::GetVarName {Section Index} {
      variable SectionList
      variable VarList

      set retval ""
      if [SectionExists $Section] {
         set secidx [GetSectionIndex $Section]
         if {$SectionList($secidx,Count) > $Index} {
            set retval $VarList($Section,$Index,Name)
         }
      }

      return $retval
   }

   ###########################################################
   # Returns value of the specified section/index            #
   ###########################################################
   proc ::Config::GetValueByIndex {Section Index} {
      variable SectionList
      variable VarList

      set retval ""
      if [SectionExists $Section] {
         set secidx [GetSectionIndex $Section]
         if {$SectionList($secidx,Count) > $Index} {
            set retval $VarList($Section,$Index,Value)
         }
      }		

      return $retval
   }

   ###########################################################
   # Returns value of the specified section variable name    #
   ###########################################################
   proc ::Config::GetValue {Section VarName} {
      variable SectionList
      variable VarList

      set retval ""
      if [SectionExists $Section] {
         set secidx [GetSectionIndex $Section]
         for {set varidx 0} {$varidx < $SectionList($secidx,Count)} {incr varidx} {
            if {$VarList($Section,$varidx,Name) == $VarName} {
               set retval $VarList($Section,$varidx,Value)
               break
            }
         }
      }

      return $retval
   }

   ###########################################################
   # Returns the number of sections currently in memory      #
   ###########################################################
   proc ::Config::GetSectionCount {} {
      variable SectionCount

      return $SectionCount
   }

   ###########################################################
   # Returns the name of a section                           #
   ###########################################################
   proc ::Config::GetSectionName {index} {
      variable SectionList
      variable SectionCount

      set retval ""
      if {$SectionCount > $index} {
         set retval $SectionList($index,Name)
      }
      return $retval
   }


   ###########################################################
   # Checks to see if a section has been defined yet         #
   # Returns 0 if not defined, 1 if already defined          #
   ###########################################################
   proc ::Config::SectionExists {SectionName} {
      variable SectionCount
      variable SectionList

      set retval 0

      for {set idx 0} {$idx <$SectionCount} {incr idx} {
         if {$SectionList($idx,Name) == $SectionName} {
            set retval 1
            break
         }
      }

      return $retval
   }

   ###########################################################
   # Adds a section definition                               #
   ###########################################################
   proc ::Config::AddSection {SectionName} {
      variable SectionCount
      variable SectionList

      set SectionList($SectionCount,Name)  $SectionName
      set SectionList($SectionCount,Count) 0
      incr SectionCount
   }

	###########################################################
   # Set a variable value in a specified section             #
	###########################################################
   proc ::Config::SetVariable {{Section} {VarName} {Value} {Comment ""}} {
      return [AddVariable $Section $VarName $Value $Comment]
   }

   ###########################################################
   # Adds a variable definition                              #
   ###########################################################
   proc ::Config::AddVariable {{Section} {VarName} {Value} {Comment ""}} {
      variable VarList
      variable SectionList
      variable SectionCount

      if {![SectionExists $Section]} {
         AddSection $Section
      }

      set VarCount [GetVarCount $Section]

      set VarList($Section,$VarCount,Name)    $VarName
      set VarList($Section,$VarCount,Value)   $Value
		set VarList($Section,$VarCount,Comment) $Comment

      set SecIdx [GetSectionIndex $Section]
      incr SectionList($SecIdx,Count)
   }

   ###########################################################
   # Open a config file and read all entries                 #
   # Returns: 1 on success, 0 on failure                     #
   ###########################################################
   proc ::Config::Read {ConfigFile} {
      variable IsLoaded 

      set Separator [GetSeparator]
		set CommentChar [GetCommentChar]
      set retval 0

      if [file exists $ConfigFile] {
         set rcfileID [open $ConfigFile r]

         set in_sect 0
         set SectionOpen  {^\[([A-Za-z][A-Za-z0-9_]*)\]}
         set SectionClose {^\..*}
         # TLV 12/11/2002 - support separator (other than =)
         #set VarPatn      {^([A-Za-z].*)[ ]*=[ ]*(.*)[ ]*}
         set VarPatn      {^([A-Za-z].*)[ ]*}
         append VarPatn $Separator
         append VarPatn {[ ]*(.*)[ ]*}

			# Variable including a comment
         # TLV 12/13/2002 - support comment char (other than #)
         # TLV 12/11/2002 - support separator (other than =)
         #set VarPatnComm  {^([A-Za-z].*)[ ]*=[ ]*(.*)[ ]*#.*}
         set VarPatnComm  {^([A-Za-z].*)[ ]*}
         append VarPatnComm $Separator
         #append VarPatnComm {[ ]*(.*)[ ]*#.*}
         append VarPatnComm {[ ]*(.*)[ ]*}
			append VarPatnComm $CommentChar
			append VarPatnComm {(.*)}

         while {[gets $rcfileID line] >= 0} {
            if [regexp $SectionOpen $line match Title] {
               # Enter section
               set in_sect 1
					set Section $Title
            }
            if [regexp $SectionClose $line match] {
               # Exit section
               set in_sect 0
               set Section ""
            }
            if {[regexp $VarPatnComm $line match Var Value Comment]  && $in_sect ==1} {
               # Assignment line (no comment)
               AddVariable $Section $Var $Value
            } elseif {[regexp $VarPatn $line match Var Value]  && $in_sect ==1} {
               # Assignment line with comment
               AddVariable $Section $Var $Value $Comment
            }
         }

         set retval 1
         set IsLoaded 1
         close $rcfileID
      }


      return $retval

   }; #Read


   ###########################################################
   # Writes the values currently defined in memory to a file #
   # returns 0 on failue, 1 on success                       #
   ###########################################################
   proc ::Config::Write {OutputFile} {
      variable VarList
      variable SectionList
      variable SectionCount

      set retval 0
      set Separator   [GetSeparator]
		set CommentChar [GetCommentChar]

      if {$OutputFile != ""} {
         set rcfileID [open $OutputFile w]
         for {set secidx 0} {$secidx < $SectionCount} {incr secidx} {
            set Section [GetSectionName $secidx]

            puts $rcfileID "\[$Section\]"
            for {set varidx 0} {$varidx < [GetVarCount $Section]} {incr varidx} {
               puts $rcfileID "$VarList($Section,$varidx,Name)$Separator$VarList($Section,$varidx,Value)\t$CommentChar$VarList($Section,$varidx,Comment)"
            }
            puts $rcfileID "."
            puts $rcfileID ""
         }

         close $rcfileID
      }

     set retval 1
     return $retval
   }

   # set up all the variables needed...
   Init

} ;#Config namespace
